function xM = MINAR1(P,eparV,n)
% xM = MINAR1(P,eparV,n)
% MINAR1 generates a count time series 'xM' of length 'n' and variables 'm'
% from a multivariate integer autoregressive (MINAR) process of order one
% based on the thinning operator (o) and multivariate Poisson innovations.
% For a time t, the vector variable of the MINAR process is defined as
%   _X_t = P o _X_{t-1} + _e_t,
% where _e_t is multivariate Poisson of length m, P is a matrix of size m x
% m, and _X_t is the vector variable at time t, _X_t =
% [X_{1,t},...,X_{m,t}]. The vector thinning operator is defined in terms
% of the scalar thinning operator. Each component i of P o X_{t-1} is
% defined as 
%    (P o _X_t)_i = sum_{j=1}^m P(i,j) o X_{j,t}
% and p o X is a random variable from the binomial distribution with X
% Bernulli trials and probability p, p o X ~ B(X,p).
%
% References:
% 1. M. Boudreault, A. Charpentier, Multivariate integer-valued autoregressive 
% models applied to earthquake counts, 2011, Arxiv
% 2. J. Franke, T.S. Rao, Multivariate First-Order Integer-Valued 
% Autoregression, Technical Report, Universit?t Kaiserslautern, 1993.
% 3. Manuel G. Scotto, Christian H. Wei?, Maria Eduarda Silva, Isabel Pereira, 
% Bivariate binomial autoregressive models, Journal of Multivariate Analysis, 
% Volume 125, March 2014, Pages 233-251, ISSN 0047-259X, http://dx.doi.org/10.1016/j.jmva.2013.12.014.
% (http://www.sciencedirect.com/science/article/pii/S0047259X14000086)
%
%     Copyright (C) 2015 Dimitris Kugiumtzis
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
%
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
%
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
ntrans = 0;

eparV = eparV(:);
[m,tmp]=size(P);
m2 = length(eparV)-1;
if m~=tmp || m~=m2
    error('The dimensions of the input variables do not match.');
end
xM = NaN(n,m);
% Set the initial conditions equal to the innovation. 
lambdaV = [eparV(1:m)-eparV(m+1); eparV(m+1)]';
for j=1:m
    xM(1,j) = poissrnd(lambdaV(j));
end
thinV = NaN(m,1);
binomV = NaN(m,1);
eM = genmultiPoisson(eparV,n+ntrans);
for t=2:n+ntrans
    for i=1:m
        for j=1:m
            binomV(j) = binornd(xM(t-1,j),P(i,j));
        end
        thinV(i)= sum(binomV);
    end
    xM(t,:) = thinV' + eM(t,:);
end
xM = xM(ntrans+1:ntrans+n,:);
